"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = __importDefault(require("."));
const chai = __importStar(require("chai"));
const chai_as_promised_1 = __importDefault(require("chai-as-promised"));
chai.use(chai_as_promised_1.default);
const { expect } = chai;
const model_1 = require("./model");
const mongodb_1 = require("mongodb");
const config = {
    database: "test",
    collection: "test_model",
    model: {
        default_name: "NONAME",
        default_age: -1,
        default_val: "DefVal"
    }
};
const model_definition = {
    name: config.collection,
    versions: [
        {
            migration: async (doc) => { },
            schema: {
                username: {
                    type: String
                },
                val: {
                    type: Number,
                    default: 0
                },
                name: {
                    type: String,
                    default: config.model.default_name
                },
                type: {
                    type: String,
                    default: "test",
                    validate: (t) => t === "test" ? undefined : "Type has to be test!"
                },
                is_male: {
                    type: Boolean,
                    optional: true
                },
                created: {
                    type: Date,
                    default: () => new Date()
                },
                nested: {
                    model: true,
                    type: {
                        val: {
                            type: String,
                            default: config.model.default_val
                        }
                    }
                },
                nested_arr: {
                    model: true,
                    array: true,
                    type: {
                        val: {
                            type: String
                        }
                    }
                }
            }
        }
    ]
};
const new_version = {
    migration: async (doc) => {
        doc.age = -1;
    },
    schema: {
        username: {
            type: String
        },
        name: {
            type: String,
            default: config.model.default_name
        },
        is_male: {
            type: Boolean,
            optional: true
        },
        created: {
            type: Date,
            default: () => new Date()
        },
        nested: {
            model: true,
            type: {
                val: {
                    type: String,
                    default: config.model.default_val
                }
            }
        },
        nested_arr: {
            model: true,
            array: true,
            type: {
                val: {
                    type: String
                }
            }
        },
        age: {
            type: Number,
            default: config.model.default_age
        }
    }
};
describe("Database", () => {
    it("connect", async () => {
        let c = new _1.default("mongodb://localhost", config.database);
        await c.connect();
        expect(c.db).to.exist;
    }).timeout(10000);
    it("await_connection", async () => {
        let c = new _1.default("mongodb://localhost", config.database);
        c.connect();
        await c.awaitConnected();
        expect(c.db).to.exist;
    });
});
describe("Model", () => {
    let safemongo;
    const dropC = () => safemongo.db.collection(config.collection).drop().catch(() => { });
    before(async () => {
        safemongo = new _1.default("mongodb://localhost", config.database);
        await safemongo.connect();
        await dropC();
    });
    it("constructor", () => {
        let model = safemongo.addModel(model_definition);
        expect(model.version).to.equal(0);
    });
    describe("Model functions", () => {
        let model;
        before(() => {
            model = safemongo.addModel(model_definition);
        });
        describe("new", () => {
            it("empty parameter", () => {
                let nd = model.new();
                expect(nd).to.exist;
                expect(nd.username).to.not.exist;
                expect(nd.name).to.equal(config.model.default_name);
                expect(nd.age).to.not.exist;
                expect(nd.is_male).to.not.exist;
                expect(nd.created).to.be.instanceof(Date);
                expect(nd.nested).to.exist;
                expect(nd.nested.val).to.equal(config.model.default_val);
            });
            it("correct parameter", () => {
                let nd = model.new({ username: "testun", nested: { val: "teeest" } });
                expect(nd).to.exist;
                expect(nd.username).to.equal("testun");
                expect(nd.name).to.equal(config.model.default_name);
                expect(nd.age).to.not.exist;
                expect(nd.is_male).to.not.exist;
                expect(nd.created).to.be.instanceof(Date);
                expect(nd.nested).to.exist;
                expect(nd.nested.val).to.equal("teeest");
            });
            it("invalid property in parameter", () => {
                const t = () => model.new({ username: "testun", invalid_prop: "asd" });
                expect(t).to.throw();
            });
            it("invalid type in parameter", () => {
                const t = () => model.new({ username: 15 });
                expect(t).to.throw();
            });
            describe("nested proprty in parameter", () => {
                it("invalid property", () => {
                    const t = () => model.new({ username: "testun", nested: { invalid_prop: "asd" } });
                    expect(t).to.throw();
                });
                it("invalid type", () => {
                    const t = () => model.new({ nested: { val: 16 } });
                    expect(t).to.throw();
                });
                it("invalid property arr", () => {
                    const t = () => model.new({ username: "testun", nested_arr: { invalid_prop: "asd" } });
                    expect(t).to.throw();
                });
            });
        });
        describe("save - new", () => {
            it("valid", async () => {
                let md = model.new({
                    is_male: true,
                    name: "Name One",
                    nested: {
                        val: "Test"
                    },
                    username: "username_1"
                });
                await model.save(md);
            });
            it("invalid", async () => {
                let md = model.new({
                    is_male: true,
                    name: "Name One",
                    nested: {
                        val: "Test"
                    }
                });
                md.username = 12;
                await expect(model.save(md)).to.eventually.be.rejected;
            });
            it("invalid - validate function", async () => {
                let md = model.new({
                    is_male: true,
                    name: "Name One",
                    nested: {
                        val: "Test"
                    },
                    username: "username_1"
                });
                md.type = "nottest";
                await expect(model.save(md)).to.eventually.be.rejected;
            });
        });
        describe("find", () => {
            before(async () => {
                await dropC();
                for (let i = 1; i < 11; i++) {
                    let m = model.new({
                        is_male: true,
                        name: "Name One",
                        nested: {
                            val: "Test"
                        },
                        username: "username_" + i
                    });
                    await model.save(m);
                }
            });
            let oneid;
            it("find one", async () => {
                let doc = await model.findOne({ username: "username_1" });
                expect(doc).to.exist;
                expect(doc._id).to.exist;
                expect(doc._id).to.be.instanceof(mongodb_1.ObjectID);
                oneid = doc._id;
            });
            it("find one - not exists", async () => {
                let doc = await model.findOne({ username: "qoiwgbfoiasbg" });
                expect(doc).to.be.null;
            });
            it("find all", async () => {
                let docs = await model.find({});
                expect(docs).has.lengthOf(10);
            });
            it("find all - not exists", async () => {
                let docs = await model.find({ username: "qoiwgbfoiasbg" });
                expect(docs).has.lengthOf(0);
            });
            it("find by id - ObjectID", async () => {
                let doc = await model.findById(oneid);
                expect(doc).to.exist;
            });
            it("find by id - string", async () => {
                let doc = await model.findById(oneid.toHexString());
                expect(doc).to.exist;
            });
            it("find by id - not exists", async () => {
                let doc = await model.findById("5be005a453b8d873d4b8cdd1");
                expect(doc).to.be.null;
            });
        });
        describe("save - change", () => {
            let test_entry;
            beforeEach(async () => {
                await dropC();
                for (let i = 1; i < 11; i++) {
                    let m = model.new({
                        is_male: true,
                        name: "Name One",
                        nested: {
                            val: "Test"
                        },
                        username: "username_" + i
                    });
                    await model.save(m);
                    test_entry = m._id;
                }
            });
            it("change property", async () => {
                let entry = await model.findById(test_entry);
                entry.name = "Name Two";
                await model.save(entry);
                entry = await model.findById(test_entry);
                expect(entry.name).to.equal("Name Two");
            });
            it("change invalid", async () => {
                let entry = await model.findById(test_entry);
                entry.name = 12;
                await expect(model.save(entry)).to.eventually.be.rejected;
                entry = await model.findById(test_entry);
                expect(entry.name).to.equal("Name One");
            });
            it("change invalid - validate function", async () => {
                let entry = await model.findById(test_entry);
                entry.type = "nottest";
                await expect(model.save(entry)).to.eventually.be.rejected;
                entry = await model.findById(test_entry);
                expect(entry.name).to.equal("Name One");
            });
            it("increment", async () => {
                let entry = await model.findById(test_entry);
                let oldval = entry.val;
                await model.incement(test_entry, "val", 1);
                entry = await model.findById(test_entry);
                expect(entry.val).to.be.equal(oldval + 1);
            });
        });
        describe("delete", () => {
            let doc;
            beforeEach(async () => {
                await dropC();
                let m = model.new({
                    is_male: true,
                    name: "Name One",
                    nested: {
                        val: "Test"
                    },
                    username: "username"
                });
                await model.save(m);
                doc = m;
            });
            it("with id", async () => {
                await model.delete(doc._id);
                let d = await model.findById(doc._id);
                expect(d).to.be.null;
            });
            it("with id string", async () => {
                await model.delete(doc._id.toHexString());
                let d = await model.findById(doc._id);
                expect(d).to.be.null;
            });
            it("with doc", async () => {
                await model.delete(doc);
                let d = await model.findById(doc._id);
                expect(d).to.be.null;
            });
            it("with filter", async () => {
                await model.deleteFilter({ username: doc.username });
                let d = await model.findById(doc._id);
                expect(d).to.be.null;
            });
        });
        it("migration", async () => {
            await dropC();
            let old_doc = model.new({
                is_male: true,
                name: "Name One",
                nested: {
                    val: "Test"
                },
                username: "username"
            });
            await model.save(old_doc);
            let nmd = model_1.recursiveDeepCopy(model_definition);
            nmd.versions.push(new_version);
            let new_model = safemongo.addModel(nmd);
            expect(new_model.version).to.be.equal(1);
            let new_doc = await new_model.findById(old_doc._id);
            expect(new_doc.username).to.be.equal(old_doc.username);
            expect(new_doc.name).to.be.equal(old_doc.name);
            expect(new_doc.is_male).to.be.equal(old_doc.is_male);
            expect(new_doc.age).to.exist;
            expect(new_doc.age).to.be.equal(config.model.default_age);
        });
    });
    after(() => safemongo.disconnect().then(() => process.exit()));
});
//# sourceMappingURL=test.js.map